#include "updater.h"
#include "ui_updater.h"

#include <QFile>

Updater::Updater(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::Updater)
{
    ui->setupUi(this);

    settingsMod = new QSettings("settings.ini", QSettings::IniFormat);
    modName = settingsMod->value("General/ModName").toString();
    Version = settingsMod->value("General/Version").toString();
    URL = settingsMod->value("General/URL").toString();

    managerSearch = new QNetworkAccessManager();
    managerDownload = new QNetworkAccessManager();

    procInstall = new QProcess;

    boxAbout = new QMessageBox;

    ui->btnDownload->setEnabled(false);
    ui->btnInstall->setEnabled(false);

    ui->textLog->append("Мод: " + modName);
    ui->textLog->append("Текущая Версия: " + Version);

    QObject::connect(ui->btnSearch, SIGNAL(clicked()), this, SLOT(Search()));
    QObject::connect(ui->btnDownload, SIGNAL(clicked()), this, SLOT(Download()));
    QObject::connect(ui->btnInstall, SIGNAL(clicked()), this, SLOT(Install()));
    QObject::connect(ui->btnAbout, SIGNAL(clicked()), this, SLOT(About()));
}

Updater::~Updater()
{
    delete ui;

    delete settingsMod;
    delete procInstall;
    delete boxAbout;
    delete managerDownload;
    delete managerSearch;
}

void Updater::Search()
{
    QString site = URL + "/update/" + Version + "/version.ini";

    QObject::connect(managerSearch, SIGNAL(finished(QNetworkReply *)), this, SLOT(onSearchResult(QNetworkReply*))); //отправляем данные и получаем ответ успешно или ошибка
    managerSearch->get(QNetworkRequest(site));
}

void Updater::onSearchResult(QNetworkReply *replyS)
{

    QFile nFile("version.ini");

    if(!replyS->error())
    {
        nFile.open(QFile::WriteOnly);
        nFile.write(replyS->readAll());
    }
    else
    {
        ui->textLog->append(replyS->errorString());
        replyS->abort();
        replyS->deleteLater();
    }

    nFile.close();

    replyS->abort();
    replyS->deleteLater();

    QString newVersion;
    QString fullSize;

    QSettings newSet("version.ini", QSettings::IniFormat);
    fullSize = newSet.value("General/Size").toString();

    newVersion = newSet.value("General/newVersion").toString();

    if (Version.toInt() < newVersion.toInt())
    {
        ui->textLog->clear();
        ui->textLog->append("Доступна новая версия!!! " + newVersion);
        ui->textLog->append("Размер: " + fullSize);

        ui->btnDownload->setEnabled(true);
    }
    else if (Version.toInt() == newVersion.toInt())
    {
        ui->textLog->clear();
        ui->textLog->append("Обновлений нет!");
    }
}

void Updater::Download()
{
    QString newVersion;

    QNetworkReply *reply;

    QSettings newSet("version.ini", QSettings::IniFormat);

    newVersion = newSet.value("General/newVersion").toString();

    QString site = URL + "/update/" + newVersion + "/update.zip";

    ui->textLog->append("Начало Загрузки...");

    reply = managerDownload->get(QNetworkRequest(site));

    QObject::connect(reply, SIGNAL(downloadProgress(qint64, qint64)), this, SLOT(onProgress(qint64, qint64)));
    QObject::connect(managerDownload, SIGNAL(finished(QNetworkReply *)), this, SLOT(onDownloadResult(QNetworkReply*))); //отправляем данные и получаем ответ успешно или ошибка
}

void Updater::onDownloadResult(QNetworkReply *replyD)
{
    QFile uFile("update.zip");

    if(!replyD->error())
    {
        uFile.open(QFile::WriteOnly);
        uFile.write(replyD->readAll());
    }
    else
    {
        ui->textLog->append(replyD->errorString());
        replyD->abort();
        replyD->deleteLater();
    }

    uFile.close();

    replyD->abort();
    replyD->deleteLater();

    ui->textLog->append("Загрузка Завершена!");
    ui->textLog->append("Файл: update.zip");
    ui->btnInstall->setEnabled(true);
}

void Updater::Install()
{
    procInstall->setProcessChannelMode(QProcess::SeparateChannels);

    procInstall->start("7za.exe", QStringList() << "x" << "-aoa" << "update.zip");

            if(!procInstall->waitForFinished())
            {
                ui->textLog->setPlainText(procInstall->errorString());
            }
            else
            {
                ui->textLog->setPlainText(procInstall->readAll());

                Version = settingsMod->value("General/Version").toString();

                system("del version.ini");
                system("del update.zip");
            }
}

void Updater::About()
{
    boxAbout->setTextFormat(Qt::RichText);
    boxAbout->setText("Update Manager"

    "\n Разработано с помощью QT Framework."
    ""
    "\n Links:"
    "\n https://qt.io"
    "\nРазработчик: Danilka Terentyev(4pda: AC97; GitHub: ac973k)"
    "");

    boxAbout->exec();
}

void Updater::onProgress(qint64 receivedSize, qint64 totalSize)
{
    ui->textLog->clear();
    ui->textLog->append("Загружено: " + QString::number(receivedSize) + " / " + QString::number(totalSize));
}

QString Updater::getName()
{
    return modName;
}
